@@ -28,6 +28,7 @@
 #include "gtr-application.h"
 #include "gtr-assistant.h"
 #include "gtr-profile.h"
+#include "gtr-profile-manager.h"
 #include "gtr-utils.h"
 #include "gtr-window.h"
 
@@ -94,7 +95,6 @@ G_DEFINE_TYPE (GtrAssistant, gtr_assistant, GTK_TYPE_ASSISTANT)
   if (l)
     {
       GList *msg_list = NULL;
-      GList *l2 = NULL;
       GFile *location;
       GError *error = NULL;
       GtrPo *po;
@@ -108,16 +108,7 @@ G_DEFINE_TYPE (GtrAssistant, gtr_assistant, GTK_TYPE_ASSISTANT)
 
       msg_list = gtr_po_get_messages (po);
 
-      for (l2 = msg_list; l2; l2 = g_list_next (l2))
-        {
-          GtrMsg *msg;
-
-          msg = GTR_MSG (l2->data);
-          if (gtr_msg_is_translated (msg))
-            gtr_translation_memory_store (data->tm,
-                                          gtr_msg_get_msgid
-                                          (msg), gtr_msg_get_msgstr (msg));
-        }
+      gtr_translation_memory_store_list (data->tm, msg_list);
 
       g_object_unref (po);
     }
@@ -167,8 +158,7 @@ destroy_idle_data (gpointer data)
 
   gtk_widget_hide (GTK_WIDGET (d->progress));
 
-  g_slist_foreach (d->list, (GFunc) g_object_unref, NULL);
-  g_slist_free (d->list);
+  g_slist_free_full (d->list, g_object_unref);
 
   gtk_widget_destroy (GTK_WIDGET (d->parent));
 
@@ -190,10 +180,11 @@ on_assistant_apply (GtkAssistant * assistant)
   GtrAssistant *as = GTR_ASSISTANT (assistant);
   const gchar *po_name;
   GtrProfile *profile;
-  GList *profiles_list;
   gulong close_signal_id;
+  GtrProfileManager *prof_manager;
 
   profile = gtr_profile_new ();
+  prof_manager = gtr_profile_manager_get_default ();
 
   gtr_profile_set_name (profile,
                         gtk_entry_get_text (GTK_ENTRY
@@ -227,16 +218,13 @@ on_assistant_apply (GtkAssistant * assistant)
                             gtk_entry_get_text (GTK_ENTRY
                                                 (as->priv->trans_enc)));
 
-  gtr_profile_set_plurals (profile,
-                           gtk_entry_get_text (GTK_ENTRY
-                                               (as->priv->plural_form)));
+  gtr_profile_set_plural_forms (profile,
+                                gtk_entry_get_text (GTK_ENTRY (as->priv->plural_form)));
 
-  gtr_application_set_active_profile (GTR_APP, profile);
+  /* Add profile to profile manager and save it */
+  gtr_profile_manager_add_profile (prof_manager, profile);
 
-  profiles_list = gtr_application_get_profiles (GTR_APP);
-
-  gtr_application_set_profiles (GTR_APP,
-                                g_list_append (profiles_list, profile));
+  g_object_unref (prof_manager);
 
   close_signal_id = g_signal_connect (as,
                                       "close",
