@@ -42,12 +42,20 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#ifdef ENABLE_INTROSPECTION
+#include <girepository.h>
+#endif
+
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#endif
+
 #define GTR_APPLICATION_GET_PRIVATE(object)	(G_TYPE_INSTANCE_GET_PRIVATE ( \
 					 (object),	\
 					 GTR_TYPE_APPLICATION,     \
 					 GtrApplicationPrivate))
 
-G_DEFINE_TYPE (GtrApplication, gtr_application, UNIQUE_TYPE_APP)
+G_DEFINE_TYPE (GtrApplication, gtr_application, GTK_TYPE_APPLICATION)
 
 struct _GtrApplicationPrivate
 {
@@ -55,7 +63,6 @@ struct _GtrApplicationPrivate
   GSettings *tm_settings;
   GSettings *window_settings;
 
-  GList *windows;
   GtrWindow *active_window;
 
   gchar *toolbars_file;
@@ -75,7 +82,7 @@ static GtrApplication *instance = NULL;
 static gboolean
 ensure_user_config_dir (void)
 {
-  gchar *config_dir;
+  const gchar *config_dir;
   gboolean ret = TRUE;
   gint res;
 
@@ -93,8 +100,6 @@ ensure_user_config_dir (void)
       ret = FALSE;
     }
 
-  g_free (config_dir);
-
   return ret;
 }
 
@@ -103,7 +108,9 @@ load_accels (void)
 {
   gchar *filename;
 
-  filename = gtr_dirs_get_user_accels_file ();
+  filename = g_build_filename (gtr_dirs_get_user_config_dir (),
+                               "accels",
+                               NULL);
   if (filename != NULL)
     {
       gtk_accel_map_load (filename);
@@ -116,7 +123,9 @@ save_accels (void)
 {
   gchar *filename;
 
-  filename = gtr_dirs_get_user_accels_file ();
+  filename = g_build_filename (gtr_dirs_get_user_config_dir (),
+                               "accels",
+                               NULL);
   if (filename != NULL)
     {
       gtk_accel_map_save (filename);
@@ -155,34 +164,27 @@ window_focus_in_event (GtrWindow      *window,
 static void
 on_window_destroy_cb (GtrWindow *window, GtrApplication *app)
 {
-  app->priv->windows = g_list_remove (app->priv->windows,
-                                      window);
+  GList *windows;
+
+  windows = gtk_application_get_windows (GTK_APPLICATION (app));
 
   if (window == app->priv->active_window)
-    set_active_window (app, app->priv->windows != NULL ? app->priv->windows->data : NULL);
-
-  if(app->priv->active_window == NULL)
-    {
-      ensure_user_config_dir ();
-      save_accels ();
-      gtk_main_quit ();
-    }
+    set_active_window (app, windows != NULL ? windows->data : NULL);
 }
 
 static void
 gtr_application_init (GtrApplication *application)
 {
-  gchar *gtr_folder;
+  GtrApplicationPrivate *priv;
+  const gchar *gtr_folder;
+  const gchar *data_dir;
   gchar *path_default_gtr_toolbar;
   gchar *profiles_file;
-  gchar *dir;
-  GtrApplicationPrivate *priv;
 
   application->priv = GTR_APPLICATION_GET_PRIVATE (application);
   priv = application->priv;
 
   priv->active_window = NULL;
-  priv->windows = NULL;
   priv->last_dir = NULL;
   priv->first_run = FALSE;
 
@@ -206,11 +208,8 @@ gtr_application_init (GtrApplication *application)
   priv->toolbars_file = g_build_filename (gtr_folder,
                                           "gtr-toolbar.xml", NULL);
 
-  g_free (gtr_folder);
-
-  dir = gtr_dirs_get_gtr_data_dir ();
-  path_default_gtr_toolbar = g_build_filename (dir, "gtr-toolbar.xml", NULL);
-  g_free (dir);
+  data_dir = gtr_dirs_get_gtr_data_dir ();
+  path_default_gtr_toolbar = g_build_filename (data_dir, "gtr-toolbar.xml", NULL);
 
   egg_toolbars_model_load_names (priv->toolbars_model,
                                  path_default_gtr_toolbar);
@@ -247,44 +246,44 @@ gtr_application_init (GtrApplication *application)
 static void
 gtr_application_dispose (GObject * object)
 {
-  GtrApplication *app = GTR_APPLICATION (object);
+  GtrApplicationPrivate *priv = GTR_APPLICATION (object)->priv;
 
   DEBUG_PRINT ("Disposing app");
 
-  if (app->priv->settings != NULL)
+  if (priv->settings != NULL)
     {
-      g_object_unref (app->priv->settings);
-      app->priv->settings = NULL;
+      g_object_unref (priv->settings);
+      priv->settings = NULL;
     }
 
-  if (app->priv->tm_settings != NULL)
+  if (priv->tm_settings != NULL)
     {
-      g_object_unref (app->priv->tm_settings);
-      app->priv->tm_settings = NULL;
+      g_object_unref (priv->tm_settings);
+      priv->tm_settings = NULL;
     }
 
-  if (app->priv->window_settings != NULL)
+  if (priv->window_settings != NULL)
     {
-      g_object_unref (app->priv->window_settings);
-      app->priv->window_settings = NULL;
+      g_object_unref (priv->window_settings);
+      priv->window_settings = NULL;
     }
 
-  if (app->priv->icon_factory != NULL)
+  if (priv->icon_factory != NULL)
     {
-      g_object_unref (app->priv->icon_factory);
-      app->priv->icon_factory = NULL;
+      g_object_unref (priv->icon_factory);
+      priv->icon_factory = NULL;
     }
 
-  if (app->priv->tm)
+  if (priv->tm)
     {
-      g_object_unref (app->priv->tm);
-      app->priv->tm = NULL;
+      g_object_unref (priv->tm);
+      priv->tm = NULL;
     }
 
-  if (app->priv->toolbars_model)
+  if (priv->toolbars_model)
     {
-      g_object_unref (app->priv->toolbars_model);
-      app->priv->toolbars_model = NULL;
+      g_object_unref (priv->toolbars_model);
+      priv->toolbars_model = NULL;
     }
 
   G_OBJECT_CLASS (gtr_application_parent_class)->dispose (object);
@@ -301,23 +300,103 @@ gtr_application_finalize (GObject *object)
   G_OBJECT_CLASS (gtr_application_parent_class)->finalize (object);
 }
 
+static void
+gtr_application_startup (GApplication *application)
+{
+  G_APPLICATION_CLASS (gtr_application_parent_class)->startup (application);
+
+  g_set_application_name (_("Gtranslator"));
+  gtk_window_set_default_icon_name ("gtranslator");
+
+  /* We set the default icon dir */
+  gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
+                                     gtr_dirs_get_gtr_pixmaps_dir ());
+}
+
+static void
+gtr_application_setup_window (GApplication *application,
+                              GFile       **files,
+                              gint          n_files)
+{
+  GtrApplicationPrivate *priv = GTR_APPLICATION (application)->priv;
+  GtrWindow *window;
+  GSList *file_list = NULL;
+
+  if (files != NULL)
+    {
+      gint i;
+      for (i = 0; i < n_files; ++i)
+        /* I don't know whether GApplication gets rid of
+         * malformed files passed on the command-line.
+         */
+        if (files[i] != NULL)
+          file_list = g_slist_prepend (file_list, files[i]);
+    }
+  window = gtr_application_create_window (GTR_APPLICATION (application));
+  gtk_application_add_window (GTK_APPLICATION (application), GTK_WINDOW (window));
+
+  /* If it is the first run, the default directory was created in this
+   * run, then we show the First run Assistant
+   */
+  if (priv->first_run)
+    gtr_show_assistant (window);
+
+  if (file_list != NULL)
+    {
+      file_list = g_slist_reverse (file_list);
+      gtr_actions_load_locations (window, file_list);
+      g_slist_free_full (file_list, g_object_unref);
+    }
+}
+
+static void
+gtr_application_open (GApplication *application,
+                      GFile       **files,
+                      gint          n_files,
+                      const gchar  *hint)
+{
+  gtr_application_setup_window (application, files, n_files);
+}
+
+static void
+gtr_application_activate (GApplication *application)
+{
+  gtr_application_setup_window (application, NULL, 0);
+}
+
+static void
+gtr_application_quit_mainloop (GApplication *application)
+{
+  ensure_user_config_dir ();
+  save_accels ();
+
+  G_APPLICATION_CLASS (gtr_application_parent_class)->quit_mainloop (application);
+}
+
 static void
 gtr_application_class_init (GtrApplicationClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (GtrApplicationPrivate));
 
   object_class->dispose = gtr_application_dispose;
   object_class->finalize = gtr_application_finalize;
+
+  application_class->startup = gtr_application_startup;
+  application_class->open = gtr_application_open;
+  application_class->activate = gtr_application_activate;
+  application_class->quit_mainloop = gtr_application_quit_mainloop;
 }
 
 GtrApplication *
 _gtr_application_new ()
 {
   instance = GTR_APPLICATION (g_object_new (GTR_TYPE_APPLICATION,
-                                            "name", "org.gnome.Gtranslator",
-                                            "startup-id", NULL, NULL));
+                                            "application-id", "org.gnome.Gtranslator",
+                                            "flags", G_APPLICATION_HANDLES_OPEN,
+                                            NULL));
 
   return instance;
 }
@@ -327,7 +406,7 @@ _gtr_application_new ()
  * 
  * Returns the default instance of the application.
  * 
- * Returns: the default instance of the application.
+ * Returns: (transfer none): the default instance of the application.
  */
 GtrApplication *
 gtr_application_get_default (void)
@@ -336,12 +415,12 @@ gtr_application_get_default (void)
 }
 
 /**
- * gtr_application_open_window:
+ * gtr_application_create_window:
  * @app: a #GtrApplication
  *
  * Creates a new #GtrWindow and shows it.
  * 
- * Returns: the #GtrWindow to be opened
+ * Returns: (transfer none):  the #GtrWindow to be opened
  */
 GtrWindow *
 gtr_application_create_window (GtrApplication *app)
@@ -355,9 +434,6 @@ gtr_application_create_window (GtrApplication *app)
   window = g_object_new (GTR_TYPE_WINDOW, NULL);
   set_active_window (app, window);
 
-  app->priv->windows = g_list_prepend (app->priv->windows,
-                                       window);
-
   state = g_settings_get_int (app->priv->window_settings,
                               GTR_SETTINGS_WINDOW_STATE);
 
@@ -388,13 +464,6 @@ gtr_application_create_window (GtrApplication *app)
 
   gtk_widget_show (GTK_WIDGET (window));
 
-  /*
-   * If it is the first run, the default directory was created in this
-   * run, then we show the First run Assistant
-   */
-  if (app->priv->first_run && app->priv->windows->next == NULL)
-    gtr_show_assistant (window);
-
   return window;
 }
 
@@ -404,7 +473,7 @@ gtr_application_create_window (GtrApplication *app)
  * 
  * Returns the toolbar model.
  * 
- * Retuns: the toolbar model.
+ * Returns: the toolbar model.
  */
 GObject *
 _gtr_application_get_toolbars_model (GtrApplication * application)
@@ -433,7 +502,7 @@ _gtr_application_save_toolbars_model (GtrApplication * application)
  *
  * Returns all the views currently present in #GtranslationApplication.
  *
- * Return value: a newly allocated list of #GtranslationApplication objects
+ * Return value: (transfer container): a newly allocated list of #GtranslationApplication objects
  */
 GList *
 gtr_application_get_views (GtrApplication * app,
@@ -455,7 +524,7 @@ gtr_application_get_views (GtrApplication * app,
  * gtr_application_get_active_window:
  * @app: a #GtrApplication
  * 
- * Return value: the active #GtrWindow
+ * Return value: (transfer none): the active #GtrWindow
  **/
 GtrWindow *
 gtr_application_get_active_window (GtrApplication * app)
@@ -465,24 +534,6 @@ gtr_application_get_active_window (GtrApplication * app)
   return GTR_WINDOW (app->priv->active_window);
 }
 
-/**
- * gtr_application_get_windows:
- * @app: a #GtrApplication
- * 
- * Return value: a list of all opened windows.
- **/
-const GList *
-gtr_application_get_windows (GtrApplication * app)
-{
-  g_return_val_if_fail (GTR_IS_APPLICATION (app), NULL);
-
-  if (!app->priv->windows)
-    app->priv->windows =
-      g_list_prepend (app->priv->windows, app->priv->active_window);
-
-  return app->priv->windows;
-}
-
 /**
  * gtr_application_register_icon:
  * @app: a #GtrApplication
@@ -497,7 +548,7 @@ gtr_application_register_icon (GtrApplication *app,
 {
   GtkIconSet *icon_set;
   GtkIconSource *icon_source;
-  gchar *pixmaps_dir;
+  const gchar *pixmaps_dir;
   gchar *path;
   GdkPixbuf *pixbuf;
 
@@ -505,9 +556,8 @@ gtr_application_register_icon (GtrApplication *app,
   g_return_if_fail (icon != NULL && stock_id != NULL);
 
   icon_source = gtk_icon_source_new ();
-  pixmaps_dir = gtr_dirs_get_pixmaps_dir ();
+  pixmaps_dir = gtr_dirs_get_gtr_pixmaps_dir ();
   path = g_build_filename (pixmaps_dir, icon, NULL);
-  g_free (pixmaps_dir);
 
   pixbuf = gdk_pixbuf_new_from_file (path, NULL);
   if (pixbuf)
@@ -549,6 +599,14 @@ _gtr_application_set_last_dir (GtrApplication * app, const gchar * last_dir)
   app->priv->last_dir = g_strdup (last_dir);
 }
 
+/**
+ * gtr_application_get_translation_memory:
+ * @app: a #GtrApplication
+ *
+ * Gets the translation memory.
+ *
+ * Returns: (transfer none): Get the translation memory.
+ */
 GObject *
 gtr_application_get_translation_memory (GtrApplication * app)
 {
